class Api::UsersController < Api::AdminBaseController
  #
  # Only admin can access these.
  #
  include Account::UsersHelper

  def show
    user = User.find_by_id(params[:user_id])

    unless user.present?
      raise Api::ContentNotFoundError,
          "No such user with ID #{params[:user_id]}",
          "No such user with ID #{params[:user_id]}"

    end

    api_hash = user.to_user_api_hash

    respond_to do |format|
      format.json { render json: api_hash }
    end
  end

  def index

    params.each { |k, v| params[k] = ActionController::Base.helpers.sanitize(v) }

    if params[:name]
      users = [User.find_by_name(params[:name])].compact
      raise Api::ContentNotFoundError if users.nil? or users.empty?
    else
      if params[:page_length].present? || params[:page_num].present?
        page_length = (params[:page_length] || 10).to_i
        page_num = (params[:page_num] || 1).to_i
        page_num = (page_num == 0 ? 1 : page_num)
        page_offset = (page_num - 1) * page_length
      elsif params[:startIndex].present? || params[:count].present?
        page_offset = (params[:startIndex] || 1).to_i  # default value ignores template user
        page_length = (params[:count] || 10).to_i
      end

      # condition u.id <> 1 is specified in query to exclude the template user
      query_string = "select u.* from users u LEFT JOIN saml_configurations s ON (u.saml_configuration_id is not NULL AND u.saml_configuration_id = s.id) WHERE u.id <> 1"
      query_string << " AND u.account_activated = #{params[:account_activated]} AND (s.enabled = #{params[:account_activated]} OR s.enabled is NULL)" if params[:account_activated].present?
      query_string << " AND u.account_expires = #{params[:account_expires]}" if params[:account_expires].present?
      query_string <<  " ORDER BY u.id"
      query_string <<  " LIMIT #{page_offset}, #{page_length};"  if page_offset && page_length

      users = User.find_by_sql(query_string)
    end

    api_hashes = users.compact.map { |u| u.to_user_api_hash }

    respond_to do |format|
      format.json { render json: api_hashes }
    end
  end

  def create
    @user = create_helper

    profile_params = params[:user_profile_data_attributes] ? Hash[params[:user_profile_data_attributes].collect { |v| [v, v] }] : nil
    @user.build_user_profile_data(profile_params)

    if params[:user_profile_data_attributes].present?
      upd_attrs = validate_user_profile_data_attributes(params[:user_profile_data_attributes])

      upd_attrs.each do |attrs|
        @user.user_profile_data << UserProfileDatum.new(
            user_profile_field_id: attrs['user_profile_field_id'],
            value: attrs['value']
        )
      end
    end

    if params[:skip_required_field_validations].present?
      @user.skip_name_validations = true
    end

    unless @user.save_with_user_profile_data_validation
      errors = "Problematic attributes:"
      conflict_error = false
      @user.errors.each{|attr,msg| errors += "  \"#{attr}\" #{msg}";
                        conflict_error = true if attr == 'name' and
                        msg.include?('already been taken')}
      if conflict_error
        raise Api::ConflictError.new(
                  "Error creating duplicate user, #{errors}")
      else
        raise Api::BadRequestError.new(
                  "Error creating user, #{errors}")
      end

    end

    api_hash = @user.to_user_api_hash
    mail_welcome_notification(@user) if params[:welcome_email]

    respond_to do |format|
      format.json { render json: api_hash }
    end
  end

  def update
    prep_params
    
    @user = params[:user_id].nil? ? ExternalUser.find_by_name(params[:name]) : User.find_by_id(params[:user_id])

    unless @user.present?
      raise Api::BadRequestError.new(
          "User with ID #{params[:id]} doesn't exist")
    end

    # get attributes that will be changed
    changed_attrs = get_changed_attrs(@user, params)

    set_user_attributes(@user, params)

    profile_params = params[:user_profile_data_attributes] ? Hash[params[:user_profile_data_attributes].collect { |v| [v, v] }] : nil
    @user.build_user_profile_data(profile_params)

    if params[:user_profile_data_attributes].present?
      upd_attrs = validate_user_profile_data_attributes(params[:user_profile_data_attributes])

      upd_attrs.each do |attrs|
        if upd = UserProfileDatum.find_by_user_id_and_user_profile_field_id(@user.id, attrs['user_profile_field_id'])
          upd.update_attribute(:value, attrs['value'])
        else
          @user.user_profile_data << UserProfileDatum.new(
              user_profile_field_id: attrs['user_profile_field_id'],
              value: attrs['value']
          )
        end
      end
    end

    if params[:skip_required_field_validations].present?
      @user.skip_name_validations = true
    end

    unless @user.save_with_user_profile_data_validation   # with_user_profile_data_validation
      errors = "Problematic attributes:"
      @user.errors.each{|attr,msg| errors += "  \"#{attr}\" #{msg}" }
      raise Api::BadRequestError.new(
                "Error updating user, #{errors}")
    end
    if changed_attrs.present?
      log_account_changes(current_user, changed_attrs, @user)
    end
    
    api_hash = @user.to_user_api_hash
    respond_to do |format|
      format.json { render json: api_hash }
    end
  end

  def delete
    raise Api::ContentNotFoundError, "User ID #{params[:user_id]} not found for deletion" unless @user = User.find_by_id(params[:user_id])

    if params[:destroy]
      @user.destroy
    else
      @user.deactivate
    end

    head :no_content
  end

  def configuration
    raise Api::ContentNotFoundError, "User ID #{params[:user_id]} not found" unless user = User.find_by_id(params[:user_id])

    respond_to do |format|
      format.json { render json: user.configuration_hash }
    end
  end

  def create_helper
    prep_params

    @user = ExternalUser.find_by_name(params[:name]) || ExternalUser.find_by_email(params[:name])
    if params[:saml_configuration_id] || params[:authorization_domain_id]
      if @user
        @user = ExternalUser.upgrade_to_remote_user(@user, params)
      else
        @user = DirectoryServiceUser.new
        if DirectoryServiceUser.first(:conditions => ["name = ?", params[:name]])
          @user.errors.add(:name, "has already been taken")
        end
        set_new_user_attributes(@user, params)
      end
    else
      if @user
        @user = ExternalUser.upgrade(@user, params)
      else
        @user = LocalUser.new
        if LocalUser.first(:conditions => ["name = ?", params[:name]])
          @user.errors.add(:name, "has already been taken")
        end
        set_new_user_attributes(@user, params)
      end
    end

    @user.role = params[:role]
    @user.can_send_normal_packages = set_account_permissions(params[:can_send_normal_packages])
    @user.can_send_to_external_email = set_account_permissions(params[:can_send_to_external_email])
    @user.can_send_to_all_faspex_users = set_account_permissions(params[:can_send_to_all_faspex_users])
    @user.creator_name = hmac_login? ? 'HMAC-authorized admin' : current_user.name

    @user
  end

  def set_account_permissions(value)
    case value.to_s
    when "true"
      User::Permissionable::YES
    when "false"
      User::Permissionable::NO
    else
      User::Permissionable::DEFAULT
    end
  end

  def prep_params
    if params[:password_expiration_interval] && params[:password_expires]
      d = Date.today + params[:password_expiration_interval].to_i
      params[:password_expiration_date] = "#{d.year}-#{d.month}-#{d.mday}"
    end

    [:expiration_date, :destruction_date].each do |date_attr|
      date_value = Utils.convert_date(params, date_attr)
      params[date_attr] = date_value unless date_value.nil?
    end
  end

  def validate_user_profile_data_attributes(data)
    data_attrs = []
    data.each do |hash|
      # Ensure both user and user profile field are valid
      raise Api::ContentNotFoundError, "No such user profile field with ID #{hash['user_profile_field_id']}" unless UserProfileField.find_by_id(hash['user_profile_field_id'])

      allowed_attributes = [
          :user_profile_field_id,
          :value
      ]

      # Make sure required fields are included
      allowed_attributes.each { |a| raise Api::BadRequestError, "Missing user profile data attribute #{a}" unless hash.keys.include?(a.to_s) }

      # Ignore any other fields
      data_attrs << hash.select{ |k, v| allowed_attributes.include?(k.to_sym) }
    end

    data_attrs
  end

  def set_new_user_attributes(user, attributes)
    set_user_attributes(user, attributes)
    user.name = attributes[:name]
  end

  def set_user_attributes(user, attributes)
    whitelist = [:account_activated,
                 :account_expires,
                 :admin,
                 :allow_user_specified_delete_after_download,
                 :allowed_ip_addresses_for_download,
                 :allowed_ip_addresses_for_login,
                 :allowed_ip_addresses_for_upload,
                 :authorization_domain_id,
                 :can_create_from_remote,
                 :can_have_public_url,
                 :can_invite,
                 :can_send_normal_packages,
                 :can_send_to_all_faspex_users,
                 :can_send_to_external_email,
                 :delete_after_download_policy,
                 :description,
                 :downloads_allowed,
                 :email,
                 :expiration_date,
                 :first_name,
                 :forwarding_allowed,
                 :hide_user_dir,
                 :home_server_id,
                 :home_server_name,
                 :include_password,
                 :kbps_initial_target_download_rate,
                 :kbps_initial_target_upload_rate,
                 :kbps_max_target_download_rate,
                 :kbps_max_target_upload_rate,
                 :last_name,
                 :lock_minrate_policy,
                 :override_home_server,
                 :override_server_delete_after_download,
                 :override_transfer_parameters,
                 :password,
                 :password_expiration_date,
                 :password_expiration_interval,
                 :password_expires,
                 :privileges,
                 :received_notification_cc,
                 :role,
                 :saml_configuration_id,
                 :type,
                 :uploads_allowed,
                 :welcome_email]
    whitelist += [:password, :password_confirmation] unless user.is_a?(DirectoryServiceUser)
    whitelist += [:self_destructs, :destruction_date] if user.is_a?(SelfRegisteredUser)
    user.set_whitelisted_attributes(whitelist, attributes)
  end

end
