require_dependency 'packages/package_methods'

class RestApiServicesController < AuthenticatedController
  include PackageMethods

  before_filter :set_delete_cookie_header
  before_filter :check_upload_allowed, :only => [:send_from_client]

  # don't count an API user as logged in unless a transfer is initiated
  skip_before_filter :set_expiry_time,
      :except => [:send_from_client, :issue_token]
  before_filter :check_home_server, :except => :signon # signon renders its own errors
  before_filter :check_send_format, :only => [:send_from_client]
  before_filter :parse_send_request,
      :only => [:send_from_client]

  skip_before_filter :basic_auth_challenge, :check_permissions, :only => :signon
  skip_before_filter :set_locale

  def set_delete_cookie_header
    response.headers["X-Delete-Cookie"] = 1
  end

  def api_request?
    true
  end

  # Service: signon
  def signon
    case
      when !current_user
        render_signon_error(401, "Invalid username and/or password", :unauthorized)
      when !current_user.local?
        render_signon_error(401, "This is not your home server", :unauthorized)
      when !current_user.login_from_ip_allowed?(request.remote_ip)
        render_signon_error(401, "Not allowed from this address", :unauthorized)
      when current_user.password_expired?
        render_signon_error(401, "Password expired", :unauthorized)
      when client_license_provided? && !client_allowed?
        render_signon_error(ErrorCodes::CLIENT_LICENSE_NOT_ALLOWED, "Client license not allowed", :ok)
      else
        render_signon_response
    end
  end

  def render_signon_error(code, user_message, status = :ok)
    render(
      :json => {
        :error => { :code => code, :user_message => user_message}
      },
      :status => status
    )
  end

  def render_signon_response
    default_profile = MetadataProfile.default.all_fields_array rescue []

    response = {}

    response[:permissions] = {
        can_send_to_external_email:
            EConfiguration.send_to_external_email_allowed? &&
                current_user.can_send_to_external_email?,
        can_send_to_all_faspex_users:
            current_user.allowed_to_send_to_all_faspex_users?,
        can_send_normal_packages:
            current_user.can_send_normal_packages?
    }

    response[:server_capabilities] = {
        supports_legacy_addressing: true
    }

    response[:default_package_profile] = {
        encryption_at_rest_required: (EConfiguration.require_encryption_at_rest == EConfiguration::REQUIRE_EAR_YES),
        encryption_at_rest_allowed: (EConfiguration.require_encryption_at_rest != EConfiguration::REQUIRE_EAR_NO),
        metadata_schema_fields: default_profile
    }

    render json: response
  end

  def client_license_provided?
    params[:license].present?
  end

 def client_allowed?
    client_license = AsperaLicense.new(params[:license])
    if EConfiguration.first.safe_net_entitlement_mode_on?
      fod_valid_with_peer_license?(client_license)
    else
      faspex_license = AsperaLicenseReader.load_license
      faspex_license.valid_with_peer_license?(client_license)
    end
  end

  def fod_valid_with_peer_license?(peer_license)
    return false if EConfiguration.first.license_keys_json.blank?
    begin
      peer_license.peer_requirements.each do |req|
        name = req[:name]
        value = req[:value]
          unless JSON.parse(EConfiguration.first.license_keys_json)[name] == value
            logger.warn("Did not meet peer requirement of #{name}=#{value}")
            return false
          end
      end
      return true
    rescue => e
      logger.warn("Error validating peer license. #{e.message} : #{e.backtrace}")
      return false
    end
  end

  # Service: send-to
  def parse_send_request
    @parsed_request = SendPackageRequest.new(current_user, request.body.string)
    unless @parsed_request.valid?
      render_send_error(@parsed_request.error_condition, @parsed_request.error_message)
    end
  end

  def send_from_client
    send_params = @parsed_request.send_params(
        skip_path_validation: true,
        skip_metadata_validation: true,
        skip_recipient_validation: EConfiguration.first.ignore_invalid_recipients
    )

    delivery = Delivery.create!(send_params)
    upload_url = get_upload_spec(delivery, true).to_fasp_url.to_s
    render_xml_package(:fasp_url => upload_url, :uuid => delivery.e_package.e_package_uuid)
  rescue => e
    handle_send_errors(delivery, e)
  end

  def handle_send_errors(delivery, exception)
    logger.info("Error sending package: #{exception}\n#{exception.backtrace.join("\n")}")
    case
      when exception.is_a?(EPackage::SendError)
        render_send_error('package-creation-error', exception.to_s)
      when delivery.nil? && /Validation failed:(.+)/.match(exception.to_s)
        render_send_error('package-creation-error', $1.strip)
      when delivery.nil?
        render_send_error('package-creation-error', 'internal server error')
      when delivery.errors_on(:recipient_list).any?
        render_send_error('invalid-recipient',
           delivery.errors_on(:recipient_list).full_messages.join("\n"))
      when exception.is_a?(EPackage::UploadFromSourceFailed)
        render_send_error('source-unreachable', 'Unable to send from remote source')
      else
        raise exception
    end
  end

  # Service: address-book

  def address_book_portable_contacts
    begin
      # Prevent filterType from being used by the rest API
      # This is for the ability to filter workgroups in order to fix #17437
      params.delete(:filterType)
      users, pagination_info, fields_to_include, format = current_user.portable_contacts(params)
    rescue ArgumentError
      Utils.log_debug($!.to_s + "\n" + $!.backtrace.join("\n"))
      return render(:xml => {:message => "Invalid contact ID"}.to_xml(:root => 'Error'), :status => 404)
    rescue NotImplementedError
      return render(:xml => {:message => $!.to_s}.to_xml(:root => 'Error'), :status => 400)
    end

    response = User.contacts_info_in_portable_contacts_format(users, pagination_info, fields_to_include, format)
    if format == 'xml'
      return render(:xml => response, :status => 200)
    elsif format == 'json'
      return render(:json => response, :status => 200)
    end
  end


  # Service: Use Inbox Feed (atom.inbox) Endpoint function for quering a User's inbox to generate via the _content.xml.builder for the atom feed
  def package_feed
    @deliveries = current_user.deliveries(params[:box], page, params[:count], params[:startIndex])
  end

  # Token issuing service
  def issue_token
    begin
      request_body = request.body.string
      requesting_user_id = current_user.id
      token_string = UrlRegeneration::Regenerator.issue_token_for_urls(request_body,
                                                                requesting_user_id,
                                                                request.remote_ip,
                                                                request.host)
      return render(:text => token_string, :content_type => 'text/plain', :status => 200)
    rescue UrlRegeneration::Errors::RegenerationError
      case $!.condition
      when 'unauthorized'
        response_code = 401
      when 'invalid-url', 'xml-not-well-formed'
        response_code = 400
      when 'url-not-found'
        response_code = 404
      else
        response_code = 500
      end
      return render(:text => $!.to_s, :content_type => 'text/plain', :status => response_code)
    end
  end

  def me
    render json: {
      username: current_user.name,
      first_name: current_user.first_name,
      last_name: current_user.last_name,
      email: current_user.email,
      downloads_allowed: current_user.downloads_allowed,
      uploads_allowed: current_user.uploads_allowed,
    }
  end

  private

  def check_upload_allowed
    unless current_user.upload_from_ip_allowed?(request.remote_ip)
      redirect_to access_denied_path  and return false
    end
  end

  def check_send_format
    unless request.format == 'application/vnd.aspera.package+xml' || request.format.xml? || request.accepts.include?('*/*')
      render_send_error('unsupported-format', "#{request.format} is not supported")
    end
  end

  def render_send_error(condition, message = nil)
    xml = Builder::XmlMarkup.new(:indent => 2)
    xml.instruct! :xml, :version=>"1.0", :encoding=>"UTF-8"
    xml.package("xmlns" => "http://schemas.asperasoft.com/xml/package") do
      xml.error do
        xml.tag!(condition)
        xml.text(message)
      end
    end
    render :xml => xml.target!, :status => :bad_request
  end

  def render_xml_package(attributes)
    builder = Builder::XmlMarkup.new(:indent => 2)
    builder.instruct! :xml, :version=>"1.0", :encoding=>"UTF-8"
    builder.package("xmlns" => 'http://schemas.asperasoft.com/xml/package') do |p|
      p.tag!(:id, attributes[:uuid])
      if attributes[:fasp_url]
        p.link("rel" => "attachments", "href" => attributes[:fasp_url])
      end
      if attributes[:delivery]
        p.link("rel" => "status", "href" => sent_url(attributes[:delivery]))
      end
    end
    render :xml => builder.target!
  end

  def check_home_server
    authenticated_user = current_user
    unless authenticated_user.nil?
      unless authenticated_user.local? # even admins are prohibited from using a home server other than their own
        reset_session
        return render(:xml => {:message => "This is not your home server."}.to_xml(:root => 'Error'), :status => 401)
      end
    end
  end

end
