class TransferOrder < ActiveRecord::BaseWithoutTable
  # General parameters
  attr_accessor :operation, :cookie, :token

  # RemoteLocation parameters
  attr_accessor :remote_address, :remote_username, :remote_password,
      :remote_ssh_port, :remote_paths

  # LocalLocation parameters
  attr_accessor :local_paths

  # ChannelParameters
  attr_accessor :remote_fasp_port

  # FileParameters
  attr_accessor :ear_passphrase, :overwrite, :source_base, :tags

  # RateParameters
  attr_accessor :target_rate_kbps, :min_rate_kbps, :max_rate_kbps, :policy,
      :priority

  # SecurityParameters
  attr_accessor :encryption_cipher

  # RetryParameters
  attr_accessor :retry_count
  default_value_for :retry_count, 3

  attr_accessible :operation, :cookie, :token, :remote_address, :remote_ssh_port,
      :remote_username, :remote_fasp_port, :remote_paths, :local_paths,
      :target_rate_kbps, :min_rate_kbps, :max_rate_kbps, :policy, :priority,
      :encryption_cipher, :ear_passphrase, :retry_count, :overwrite, :source_base, :remote_password, :tags

  validates_presence_of :cookie, :remote_address, :remote_username,
      :remote_paths, :local_paths

  validates_inclusion_of :operation, :in => [:push, :pull]
  validates_inclusion_of :policy, :in => [:fixed, :fair, :trickle]

  # validates_numericality_of doesn't work with attr_accessor (only with column)
  # keeping attr_accessor for forward compatibility with Rails 3 ActiveModel
  must_be_positive = [:remote_ssh_port, :target_rate_kbps, :remote_fasp_port,
      :min_rate_kbps, :priority]
  validates_each(must_be_positive) do |record, attr, value|
    unless value.is_a?(Fixnum) && value >= 0
      record.errors.add(attr, 'must be a positive integer')
    end
  end

  def to_soap
    soap_xml.to_s
  end

  # SOAP generation code from Console below

  def soap_xml
    REXML::Document.new.tap do |xml|
      xml << REXML::XMLDecl.new
      root = xml.add_element('soapenv:Envelope',
          'xmlns:soapenv' => "http://schemas.xmlsoap.org/soap/envelope/")
      root << element('soapenv:Header')
      root << el_SoapBody
    end
  end

  def el_SoapBody
    element('soapenv:Body').tap do |soap_body|
      submit_request = soap_body.add_element('ns1:SubmitRequest',
          'xmlns:ns1' => "urn:Aspera:XML:JobNET:2006/01:Types")
      submit_request << element('Types', 'Aspera.IScpTransfer')
      definition = submit_request << element('Definition')
      definition.text = el_Order.to_s
    end
  end

  # SAMPLE TRANSFER ORDER
  # ---------------------------------
  #  <Order>
  #      <Agent>Aspera.Faspex</Agent>
  #      <WidgetData>
  #          <Authorize>No</Authorize>
  #          <Authenticate>No</Authenticate>
  #      </WidgetData>
  #      <ApplicationData>
  #          <Cookie>Aspera.Faspex:315e3e8b-3048-4317-892e-77aac7a317f2:console1</Cookie>
  #          <Token>ATV7_XXXX_7VTA
  #      </ApplicationData>
  #      <Operation>Push</Operation>
  #      <RemoteLocation>
  #          <System>
  #              <Address>10.0.62.2</Address>
  #              <Port>22</Port>
  #              <UserName>root</UserName>
  #          </System>
  #          <Authentication>
  #              <Methods>
  #                  <Method>Token</Method>
  #              </Methods>
  #          </Authentication>
  #          <Files>
  #              <Path>/tmp</Path>
  #          </Files>
  #      </RemoteLocation>
  #      <LocalLocation>
  #          <Files>
  #              <Path>/temp/20M</Path>
  #          </Files>
  #      </LocalLocation>
  #      <FileParameters>
  #          <PreserveTimestamps>Yes</PreserveTimestamps>
  #          <Overwrite>Always</Overwrite>
  #          <CreatePath>yes</CreatePath>
  #          <ResumeCheck>None</ResumeCheck>
  #          <PreCalculateJobSize>yes</PreCalculateJobSize>
  #          <RemoveAfterTransfer>no</RemoveAfterTransfer>
  #          <RemoveEmptyDirectories>no</RemoveEmptyDirectories>
  #          <Protection>None|Encrypt|Decrypt</Protection>
  #          <Passphrase>blah</Passphrase>
  #      </FileParameters>
  #      <LinkParameters>
  #          <RemoteCapacity>2000000</RemoteCapacity>
  #          <LocalCapacity>2000000</LocalCapacity>
  #          <AutoDetectCapacity>No</AutoDetectCapacity>
  #      </LinkParameters>
  #      <ChannelParameters>
  #          <DatagramSize/>
  #          <AutoDetectPathMTU>Yes</AutoDetectPathMTU>
  #      </ChannelParameters>
  #      <Version>2</Version>
  #      <RateParameters>
  #          <TargetRate>1000</TargetRate>
  #          <TargetRateAsPercent>No</TargetRateAsPercent>
  #          <MinimumRate>0</MinimumRate>
  #          <MinimumRateAsPercent>No</MinimumRateAsPercent>
  #          <Policy>Fair</Policy>
  #          <Priority>2</Priority>
  #          <BandwidthCap>2000000</BandwidthCap>
  #      </RateParameters>
  #      <SecurityParameters>
  #          <EncryptionCipher>NONE</EncryptionCipher>
  #      </SecurityParameters>
  #      <RetryParameters>
  #          <Count>3</Count>
  #          <BaseInterval>30</BaseInterval>
  #          <MaximumInterval>30</MaximumInterval>
  #      </RetryParameters>
  #      <PrivateData>
  #          <Key/>
  #      </PrivateData>
  #  </Order>
  def el_Order
    element('Order').tap do |order|
      order << el_Agent
      order << el_ApplicationData
      order << el_Operation
      order << el_RemoteLocation
      order << el_LocalLocation
      order << el_FileParameters
      order << el_ChannelParameters
      order << el_RateParameters
      order << el_SecurityParameters
      order << el_RetryParameters
      order << el_PrivateData
    end
  end

  def el_Agent
    element('Agent', 'Aspera.Faspex')
    element('NoMgmtPortFiles','Yes')
  end

  def el_WidgetData
    element("WidgetData").tap do |wd|
      wd << element("Authorize", 'No')
      wd << element("Authenticate", 'No')
    end
  end

  def el_ApplicationData
    element('ApplicationData').tap do |ad|
      ad << element('Cookie', cookie)
      ad << element('Token', token) if token
    end
  end

  def el_Operation
    element('Operation', operation.to_s.camelcase)
  end

  def el_RemoteLocation
    element('RemoteLocation').tap do |rl|
      rl << el_RemoteLocation_System
      if remote_password.nil?
        rl << el_RemoteLocation_Authentication
      else
        rl << el_RemoteLocation_Authentication_Password
      end
      rl << el_RemoteLocation_Files
    end
  end

  def el_RemoteLocation_System
    element('System').tap do |s|
      s << element('Address',  remote_address)
      s << element('Port',     remote_ssh_port)
      s << element('UserName', remote_username)
      s << element('Password', remote_password) if !remote_password.nil?
    end
  end

  def el_RemoteLocation_Authentication
    element('Authentication').tap do |auth|
      methods = element('Methods')
      methods << element('Method', 'Token')
      auth << methods
    end
  end

  def el_RemoteLocation_Authentication_Password
    element('Authentication').tap do |auth|
      methods = element('Methods')
      methods << element('Method', 'Password')
      auth << methods
    end
  end

  def el_RemoteLocation_Files
    el_Files(remote_paths)
  end

  def el_LinkParameters
    element('LinkParameters').tap do |lp|
      lp << element('RemoteCapacity', 2_000_000)
      lp << element('LocalCapacity', 2_000_000)
      lp << element('AutoDetectCapacity', 'No')
    end
  end

  def el_ChannelParameters
    element('ChannelParameters').tap do |cp|
      cp << element('Port', remote_fasp_port)
      cp << element('DatagramSize', 1492)
      cp << element('AutoDetectPathMTU', 'Yes')
    end
  end

  def el_LocalLocation
    element('LocalLocation').tap do |ll|
      ll << el_LocalLocation_Files
    end
  end

  def el_LocalLocation_Files
    el_Files(local_paths)
  end

  def el_Files(paths)
    element('Files').tap do |f|
      paths.each do |path|
        f << element('Path', path.strip)
      end
    end
  end

  def el_FileParameters
    element('FileParameters').tap do |fp|
      fp << element('PreserveTimestamps', 'Yes')
      fp << element('CreatePath', 'Yes')
      fp << element('ResumeCheck', 'Sparse Checksum')
      fp << element('PreCalculateJobSize', 'Yes')
      fp << element('RemoveAfterTransfer', 'No')
      fp << element('RemoveEmptyDirectories', 'No')
      fp << element('SourceBase', source_base) if source_base

      if overwrite
        fp << element('Overwrite', overwrite)
      else
        fp << element('Overwrite', 'Always')
      end

      if ear_passphrase.present?
        fp << element('Protection', 'Encrypt')
        fp << element('Passphrase', ear_passphrase)
      end

      fp << element('Tags', tags) if tags.present?
    end
  end

  def el_RateParameters
    element('RateParameters').tap do |rp|
      rp << element('TargetRate', target_rate_kbps)
      rp << element('TargetRateAsPercent', 'No')
      rp << element('MinimumRate', min_rate_kbps)
      rp << element('MinimumRateAsPercent', 'No')
      rp << element('Policy', policy.capitalize)
      rp << element('Priority', priority)
      rp << element('BandwidthCap', max_rate_kbps) if max_rate_kbps
    end
  end

  def el_SecurityParameters
    element('SecurityParameters').tap do |sp|
      sp << element('EncryptionCipher',
          encryption_cipher ? (encryption_cipher.downcase == 'none' ? encryption_cipher.capitalize : encryption_cipher.upcase) : 'None')
    end
  end

  def el_RetryParameters
    element('RetryParameters').tap do |rp|
      rp << element('Count', retry_count)
      rp << element('BaseInterval', 30)
      rp << element('MaximumInterval', 30)
    end
  end

  def el_PrivateData
    element('PrivateData').tap do |pd|
      pd << element('Key')
    end
  end

  def element(name, text = nil)
    REXML::Element.new(name).tap do |el|
      el.text = text if text
    end
  end

end
